@@ -73,6 +73,8 @@ static gboolean              gail_button_set_image_description
                                                          const gchar    *description);
 static GtkImage*             get_image_from_button      (GtkWidget      *button);
 static GtkWidget*            get_label_from_button      (GtkWidget      *button);
+static void                  set_role_for_button        (AtkObject      *accessible,
+                                                         GtkWidget      *button);
 
 static GailContainer* parent_class = NULL;
 
@@ -167,7 +169,7 @@ gail_button_new (GtkWidget *widget)
   accessible = ATK_OBJECT (object);
   atk_object_initialize (accessible, widget);
 
-  accessible->role = ATK_ROLE_PUSH_BUTTON;
+  set_role_for_button (accessible, widget);
 
   return accessible;
 }
@@ -745,7 +747,7 @@ get_label_from_button (GtkWidget *button)
   child = gtk_bin_get_child (GTK_BIN (button));
   if (GTK_IS_ALIGNMENT (child))
     child = gtk_bin_get_child (GTK_BIN (child));
-  if (GTK_IS_CONTAINER (child))
+  while (GTK_IS_CONTAINER (child))
     {
       /*
        * Child is not a label.  Search for a label in the container's children.
@@ -762,9 +764,46 @@ get_label_from_button (GtkWidget *button)
 	      child = GTK_WIDGET (tmp_list->data);
 	      break;
 	    }  
+           /*
+            * Label for button which are GtkreeView column headers are in a 
+            * GtkHBox in a GtkAlignment.
+            */
+          if (GTK_IS_ALIGNMENT (tmp_list->data))
+            {
+              child = gtk_bin_get_child (GTK_BIN (tmp_list->data));
+              break;
+	    }
 	}
 
       g_list_free (children);
     }
   return child;
 }
+
+static void
+set_role_for_button (AtkObject *accessible,
+                     GtkWidget *button)
+{
+  GtkWidget *parent;
+  AtkRole role;
+
+  parent = gtk_widget_get_parent (button);
+  if (GTK_IS_TREE_VIEW (parent))
+    {
+      role = ATK_ROLE_TABLE_COLUMN_HEADER;
+      /*
+       * Even though the accessible parent of the column header will
+       * be reported as the table because the parent widget of the
+       * GtkTreeViewColumn's button is the GtkTreeView we set
+       * the accessible parent for column header to be the table
+       * to ensure that atk_object_get_index_in_parent() returns
+       * the correct value; see gail_widget_get_index_in_parent().
+       */
+      atk_object_set_parent (accessible, gtk_widget_get_accessible (parent));
+    }
+  else
+    role = ATK_ROLE_PUSH_BUTTON;
+
+  accessible->role =  role;
+}
+
