@@ -282,6 +282,16 @@ gail_entry_real_notify_gtk (GObject		*obj,
 
   if (strcmp (pspec->name, "cursor-position") == 0)
     {
+      entry = GAIL_ENTRY (atk_obj);
+
+      if (entry->signal_name)
+        {
+          g_signal_emit_by_name (atk_obj, 
+                                 entry->signal_name,
+                                 entry->position,
+                                 entry->length);
+          entry->signal_name = NULL;
+        }
       if (check_for_selection_change (entry, gtk_entry))
         g_signal_emit_by_name (atk_obj, "text_selection_changed");
       /*
@@ -1010,7 +1020,7 @@ static void _gail_entry_insert_text_cb (GtkEntry *entry,
       gail_entry->length = arg2;
     }
   /*
-   * The signal will be emitted when the changed signal is received
+   * The signal will be emitted when the cursor position is updated.
    */
 }
 
@@ -1070,18 +1080,10 @@ _gail_entry_changed_cb (GtkEntry *entry)
 
   gail_entry = GAIL_ENTRY (accessible);
 
-  if (gail_entry->signal_name)
-    {
-      g_signal_emit_by_name (accessible, 
-                             gail_entry->signal_name,
-                             gail_entry->position,
-                             gail_entry->length);
-      gail_entry->signal_name = NULL;
-    }
   /*
    * Update the text in the GtkTextBuffer
    */
-  gail_text_util_text_setup (gail_entry->textutil, gtk_entry_get_text(entry));
+  gail_text_util_text_setup (gail_entry->textutil, gtk_entry_get_text (entry));
 }
 
 static gboolean 
