@@ -152,6 +152,7 @@ static gint             get_insert_offset              (GtkTextBuffer    *buffer
 static gint             get_selection_bound            (GtkTextBuffer    *buffer);
 static void             emit_text_caret_moved          (GailTextView     *gail_text_view,
                                                         gint             insert_offset);
+static gint             insert_idle_handler            (gpointer         data);
 
 static GailWidgetClass *parent_class = NULL;
 
@@ -235,6 +236,7 @@ gail_text_view_init (GailTextView      *text_view)
   text_view->signal_name = NULL;
   text_view->previous_insert_offset = -1;
   text_view->previous_selection_bound = -1;
+  text_view->insert_notify_handler = 0;
 }
 
 static void
@@ -271,6 +273,9 @@ gail_text_view_finalize (GObject            *object)
   GailTextView *text_view = GAIL_TEXT_VIEW (object);
 
   g_object_unref (text_view->textutil);
+  if (text_view->insert_notify_handler)
+    gtk_idle_remove (text_view->insert_notify_handler);
+
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -1383,13 +1388,42 @@ _gail_text_view_insert_text_cb (GtkTextBuffer *buffer,
   GtkTextView *text = (GtkTextView *) user_data;
   AtkObject *accessible;
   GailTextView *gail_text_view;
+  gint position;
+  gint length;
+
+  g_return_if_fail (arg3 > 0);
 
   accessible = gtk_widget_get_accessible(GTK_WIDGET(text));
   gail_text_view = GAIL_TEXT_VIEW (accessible);
 
   gail_text_view->signal_name = "text_changed::insert";
-  gail_text_view->position = gtk_text_iter_get_offset (arg1);
-  gail_text_view->length = arg3;
+  position = gtk_text_iter_get_offset (arg1);
+  length = arg3;
+  
+  if (gail_text_view->length == 0)
+    {
+      gail_text_view->position = position;
+      gail_text_view->length = length;
+    }
+  else if (gail_text_view->position + gail_text_view->length == position)
+    {
+      gail_text_view->length += length;
+    }
+  else
+    {
+      /*
+       * We have a non-contiguous insert so report what we have
+       */
+      if (gail_text_view->insert_notify_handler)
+        {
+          gtk_idle_remove (gail_text_view->insert_notify_handler);
+        }
+      gail_text_view->insert_notify_handler = 0;
+      insert_idle_handler (gail_text_view);
+      gail_text_view->position = position;
+      gail_text_view->length = length;
+    }
+    
   /*
    * The signal will be emitted when the changed signal is received
    */
@@ -1414,6 +1448,24 @@ _gail_text_view_delete_range_cb (GtkTextBuffer *buffer,
 
   accessible = gtk_widget_get_accessible(GTK_WIDGET(text));
   gail_text_view = GAIL_TEXT_VIEW (accessible);
+  if (gail_text_view->insert_notify_handler)
+    {
+      gtk_idle_remove (gail_text_view->insert_notify_handler);
+      gail_text_view->insert_notify_handler = 0;
+      if (gail_text_view->position == offset && 
+          gail_text_view->length == length)
+        {
+        /*
+         * Do not bother with insert and delete notifications
+         */
+          gail_text_view->signal_name = NULL;
+          gail_text_view->position = 0;
+          gail_text_view->length = 0;
+          return;
+        }
+
+      insert_idle_handler (gail_text_view);
+    }
   g_signal_emit_by_name (accessible, "text_changed::delete",
                          offset, length);
 }
@@ -1491,11 +1543,11 @@ _gail_text_view_changed_cb (GtkTextBuffer *buffer,
   gail_text_view = GAIL_TEXT_VIEW (accessible);
   if (gail_text_view->signal_name)
     {
-      g_signal_emit_by_name (accessible,
-                             gail_text_view->signal_name,
-                             gail_text_view->position,
-                             gail_text_view->length);
-      gail_text_view->signal_name = NULL;
+      if (!gail_text_view->insert_notify_handler)
+        {
+          gail_text_view->insert_notify_handler = gtk_idle_add (insert_idle_handler, accessible);
+        }
+      return;
     }
   emit_text_caret_moved (gail_text_view, get_insert_offset (buffer));
   gail_text_view->previous_selection_bound = get_selection_bound (buffer);
@@ -1553,6 +1605,16 @@ static void
 emit_text_caret_moved (GailTextView *gail_text_view,
                        gint          insert_offset)
 {
+  /*
+   * If we have text which has been inserted notify the user
+   */
+  if (gail_text_view->insert_notify_handler)
+    {
+      gtk_idle_remove (gail_text_view->insert_notify_handler);
+      gail_text_view->insert_notify_handler = 0;
+      insert_idle_handler (gail_text_view);
+    }
+
   if (insert_offset != gail_text_view->previous_insert_offset)
     {
       /*
@@ -1565,3 +1627,32 @@ emit_text_caret_moved (GailTextView *gail_text_view,
       gail_text_view->previous_insert_offset = insert_offset;
     }
 }
+
+static gint
+insert_idle_handler (gpointer data)
+{
+  GailTextView *gail_text_view;
+  GtkTextBuffer *buffer;
+
+  gail_text_view = GAIL_TEXT_VIEW (data);
+
+  g_signal_emit_by_name (data,
+                         gail_text_view->signal_name,
+                         gail_text_view->position,
+                         gail_text_view->length);
+  gail_text_view->signal_name = NULL;
+  gail_text_view->position = 0;
+  gail_text_view->length = 0;
+
+  buffer = gail_text_view->textutil->buffer;
+  if (gail_text_view->insert_notify_handler)
+    {
+    /*
+     * If called from idle handler notify caret moved
+     */
+      gail_text_view->insert_notify_handler = 0;
+      emit_text_caret_moved (gail_text_view, get_insert_offset (buffer));
+      gail_text_view->previous_selection_bound = get_selection_bound (buffer);
+    }
+  return FALSE;
+}
